//
//  ViewController.m
//  YKPrintDemo
//
//  Created by YK on 2022/5/5.
//

#import "ViewController.h"
#import <YKPrinterKit/YKPrinterKit.h>
#import "YKFuncCollectionViewCell.h"
#import <objc/runtime.h>
#import <YKImageProcKit/YKImageProcKit.h>

#define YKBLEMANAGER [YKPrintManager sharedInstance]

@interface ViewController ()
<
    UITableViewDelegate,
    UITableViewDataSource,
    UICollectionViewDelegate,
    UICollectionViewDataSource
>
@property (weak, nonatomic) IBOutlet UITableView *tableView;
@property (weak, nonatomic) IBOutlet UICollectionView *collectionView;
@property (weak, nonatomic) IBOutlet NSLayoutConstraint *statusBarLayoutHeight;
@property (weak, nonatomic) IBOutlet UILabel *printerInfoLabel;

@property (nonatomic, strong) YKPrinter *printer;
@property (nonatomic, assign) NSInteger printCount;
@property (nonatomic, assign) NSInteger pageIndex;
@property (nonatomic, strong) NSArray *dataArray;
@end

@implementation ViewController

- (void)viewDidLoad {
    [super viewDidLoad];

    self.printCount = 0;
    self.statusBarLayoutHeight.constant = [UIApplication sharedApplication].statusBarFrame.size.height;
    self.view.backgroundColor = [UIColor colorWithRed:250.f / 255.f green:250.f / 255.f blue:250.f / 255.f alpha:1.f];
    CGFloat width = (self.view.frame.size.width - 25) / 4.f;
    UICollectionViewFlowLayout *layout = [[UICollectionViewFlowLayout alloc] init];
    layout.sectionInset = UIEdgeInsetsMake(5, 5, 5, 5);
    layout.minimumLineSpacing = 5;
    layout.minimumInteritemSpacing = 5;
    layout.itemSize = CGSizeMake(width, 50);
    self.collectionView.collectionViewLayout = layout;
    self.collectionView.backgroundColor = UIColor.clearColor;

    [self.collectionView registerNib:[UINib nibWithNibName:@"YKFuncCollectionViewCell" bundle:nil] forCellWithReuseIdentifier:@"YKFuncCollectionViewCell"];
    self.tableView.tableFooterView = [UIView new];

    // 监听通知
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(updatePrinterList) name:YK_FOUND_NEW_PRINTER_NOTIFICATION_NAME object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(printConnectStateDidChanged:) name:YK_PRINTER_CONNECT_STATE_CHANGE_NOTIFICATION_NAME object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(syncPrinterInfo:) name:YK_SYNC_PRINTER_STATE_NOTIFICATION_NAME object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(printerPrintState:) name:YK_UPDATE_PRINT_STATE_NOTIFICATION_NAME object:nil];
}

#pragma mark - TableView Delegate / Datasource
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {return 1;}
- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return YKBLEMANAGER.printerArray.count;
}
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"CellId"];
    if (!cell) {
        cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleSubtitle reuseIdentifier:@"CellId"];
    }
    YKPrinter *printer = YKBLEMANAGER.printerArray[indexPath.row];
//    NSLog(@"printer = %p  mac = %@",printer, printer.mac);
    cell.textLabel.text = printer.name;
    cell.detailTextLabel.text = [NSString stringWithFormat:@"MAC:%@ connect state = %ld",printer.mac, (long)printer.connectState];
    if (printer.connectState == YKPrinterConnectionState_Connected) {
        cell.textLabel.textColor = UIColor.greenColor;
        cell.detailTextLabel.textColor = UIColor.greenColor;
    } else {
        cell.textLabel.textColor = UIColor.blackColor;
        cell.detailTextLabel.textColor = UIColor.blackColor;
    }
    return cell;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
    YKPrinter *printer = YKBLEMANAGER.printerArray[indexPath.row];
    if (!printer) return;
    if (printer.connectState == YKPrinterConnectionState_Connecting) return;
    if (printer.connectState == YKPrinterConnectionState_Connected) {
        [YKBLEMANAGER disconnectPrinter];
        return;
    }
    
    [YKBLEMANAGER connectPrinter:printer];
    
//    if (self.printer != printer && self.printer) {
//        self.printer.connectState = YKPrinterConnectionState_Disconnected;
//        [YKBLEMANAGER disconnectPrinter];
//    } else {
//
//    }
    
    self.printer = printer;
}

#pragma mark - CollectionView Delegate / Datasource
- (NSInteger)numberOfSectionsInCollectionView:(UICollectionView *)collectionView {return 1;}
- (NSInteger)collectionView:(UICollectionView *)collectionView numberOfItemsInSection:(NSInteger)section {
    return [self funcNames].count;
}
- (__kindof UICollectionViewCell *)collectionView:(UICollectionView *)collectionView cellForItemAtIndexPath:(NSIndexPath *)indexPath {
    YKFuncCollectionViewCell *cell = (YKFuncCollectionViewCell *)[collectionView dequeueReusableCellWithReuseIdentifier:@"YKFuncCollectionViewCell" forIndexPath:indexPath];
    NSString *name = [self funcNames][indexPath.row];
    cell.funcName.text = name;
    return cell;
}

- (void)collectionView:(UICollectionView *)collectionView didSelectItemAtIndexPath:(NSIndexPath *)indexPath {
    /** 这里功能对应[self funcNames]数组中的顺序 */
    switch (indexPath.row) {
        case 0: {
//            UIImage *dddddd = [self scaleImage:[UIImage imageNamed:@"liuyifei"] andWidth:YKPrinterWidthDot_2748];
//            UIImage *printImg = [[YKImageProcKit sharedInstance] printImageProc:dddddd];
//            NSData *data = [YKBLEMANAGER imageToZebraInstruction:printImg printDensity:7 autoCout:YES];
//            NSLog(@"data= %@",data);
//            NSString *str = [[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding];
//            [self writeDataToFile:data fileName:@"dfff.txt"];
//            NSLog(@"xxx = %@",str);
            
//            self.dataArray = @[@"liuyifei",@"XingG"];
//            self.printCount = 1;
//            self.pageIndex = 0;
//            [self printImage];
            
//            [YKBLEMANAGER printLabelImageWithFeedback:[UIImage imageNamed:@"printImg"] density:YKPrinterDensity_Middle paperWidth:40 printCount:2];
//            UIImage *img = [self scaleImage:[UIImage imageNamed:@"math_test"] andWidth:self.printer.widthDot];
//            UIImage *printImg = [[YKImageProcKit sharedInstance] printImageProc:img];
            UIImage *printImg = [self scaleImage:[UIImage imageNamed:@"printImg13456"] andWidth:self.printer.widthDot];
            printImg = [[YKImageProcKit sharedInstance] printTextProc:printImg option:YKTextProc_Normal constant:-1 boldValue:-1];
            [YKBLEMANAGER printImage:printImg density:YKPrinterDensity_High count:1 labelState:NO whitespace:YES];
            
            
            // tp6
//            [YKBLEMANAGER printLabelImageForTP6:[UIImage imageNamed:@"tp6_30"] density:YKPrinterDensity_High labelWidth:30 printCount:2];
            
        }
            break;
        case 1: {
            UIImage *img = [self scaleImage:[UIImage imageNamed:@"txtDoc"] andWidth:self.printer.widthDot];
            UIImage *img1 = [self scaleImage:[UIImage imageNamed:@"XingG"] andWidth:self.printer.widthDot];
            UIImage *printImg = [[YKImageProcKit sharedInstance] printTextProc:img threshold:-1];
            UIImage *printImg1 = [[YKImageProcKit sharedInstance] printTextProc:img1 threshold:-1];
            [YKBLEMANAGER printImageArry:@[printImg, printImg1] density:YKPrinterDensity_Middle count:1 labelState:NO whitespace:YES];
        }
            break;
        case 2: {
            /** 不支持降版本 V20.250_aes和V20.251_aes是lp2的固件*/
            NSString *firmwareFile = [[NSBundle mainBundle] pathForResource:@"V20.06_aes" ofType:@"bin"];
            NSData *fileData = [NSData dataWithContentsOfFile:firmwareFile];
            [YKBLEMANAGER printerUpateFirmware:fileData processing:^(CGFloat percent) {
                NSLog(@"percent = %.2f",percent);
            } complete:^(BOOL bSuccess) {
                if (bSuccess) {
                    NSLog(@"固件升级成功.....");
                    // a4机器除外，其余机器最好能在固件升级成功后获取一次固件版本
                } else {
                    NSLog(@"固件升级失败.....");
                }
            }];
        }
            break;
        case 3: {
            [YKBLEMANAGER printWhiteSpace:100];
            
        }
            break;
        case 4: {
            [YKBLEMANAGER setPrinterAutoOffTime:120];
        }
            break;
        case 5: {
            /*
             4寸机器中放3寸纸。在打印前设置一次纸张尺寸（每次切换纸设置一次或每次打印前设置一次都行）
             相对应的要把打印的宽度设置为纸张大小
             */
            self.printer.widthDot = YKPrinterWidthDot_576;
            [YKBLEMANAGER setPrinterPaperSize:YKTP4PaperSizeInch2];
            break;
        }
        case 6: {
            self.printer.widthDot = YKPrinterWidthDot_864;
            [YKBLEMANAGER setPrinterPaperSize:YKTP4PaperSizeInch3];
            break;
        }
        case 7: {
            self.printer.widthDot = YKPrinterWidthDot_1280;
            [YKBLEMANAGER setPrinterPaperSize:YKTP4PaperSizeInch4];
            break;
        }
        case 8: {
            [YKBLEMANAGER lp2PaperInfo:^(NSDictionary * _Nonnull dict) {
                NSLog(@"info dict = %@",dict);
            }];
            break;
        }
        case 9: {
            [YKBLEMANAGER printerFirmwareInfo];
            break;
        }
        case 10: {
            [YKBLEMANAGER disconnectPrinter];
            break;
        }
        case 11: {
            [YKBLEMANAGER checkConsumablesInfo:^(NSString * _Nonnull infoStr) {
                NSLog(@"info str = %@",infoStr);
            }];
            break;
        }
        default:
            break;
    }
}

- (void)printImage {
    UIImage *img = [self scaleImage:[UIImage imageNamed:self.dataArray[self.pageIndex]] andWidth:self.printer.widthDot];
    UIImage *printImg = [[YKImageProcKit sharedInstance] printImageProc:img];
    [YKBLEMANAGER printImage:printImg density:YKPrinterDensity_Middle count:1 labelState:NO whitespace:YES];
    
//    [YKBLEMANAGER printImageArry:@[printImg] density:YKPrinterDensity_Middle count:1 labelState:NO whitespace:YES];
    
//    UIImage *img = [self scaleImage:[UIImage imageNamed:@"liuyifei"] andWidth:self.printer.widthDot];
//    UIImage *printImg = [[YKImageProcKit sharedInstance] printImageProc:img];
    /** lp2-s 连续纸打印 */
//            [YKBLEMANAGER lp2ContinuousPrintImage:printImg density:YKPrinterDensity_Middle printCount:1];
    
    /** LP2-S 标签纸打印 */
//            [YKBLEMANAGER printLabelImageWithFeedback:printImg density:YKPrinterDensity_Middle paperWidth:30 printCount:1];
    
    /** LP1-S 连续纸打印 普通单张图打印 */
//    [YKBLEMANAGER printImage:printImg density:YKPrinterDensity_Middle count:2 labelState:NO whitespace:YES];
    
    /** LP1-S 标签纸打印 */
//            [YKBLEMANAGER labelPrintImage:printImg density:YKPrinterDensity_Middle paperWidth:40 printCount:1];
    
    /** LP1-S 连续纸打印  */
}

#pragma mark - Action
- (IBAction)onClickScanButton:(UIButton *)sender {
//    [YKBLEMANAGER scanForPrinterWithDuration:5];
//    [YKBLEMANAGER.printerArray removeAllObjects];
//    [self.tableView reloadData];
    [YKBLEMANAGER clearPrinterList];
    [YKBLEMANAGER scanForPrinter:@[@(YKPrinterTypeTRP1),@(YKPrinterType200DPI),@(YKPrinterType300DPI),@(YKPrinterTypeTP2),@(YKPrinterTypeTP2_SZ),@(YKPrinterTypeTP6)] duration:5 scanEnd:nil];
//    [YKBLEMANAGER scanForPrinter:@[@(YKPrinterTypeTP200)] duration:5 scanEnd:nil];
}

#pragma mark - Notify
- (void)updatePrinterList {
    [self.tableView reloadData];
}
- (void)printConnectStateDidChanged:(NSNotification *)notify {
    YKPrinterConnectionState state = [[notify object] integerValue];
    if (state == YKPrinterConnectionState_Connected) {
        [self showConnectedPrinterInfo];
    }
    [self.tableView reloadData];
}

- (void)syncPrinterInfo:(NSNotification *)notify {
    [self showConnectedPrinterInfo];
}

- (void)printerPrintState:(NSNotification *)notify {
    YKPrinterPrintState state = [[notify object] integerValue];
    if (state != YKPrinterPrintState_PrintEnd) return;
    
//    [self testPrintMultipleCopiesAndCount];
}

//测试打印多张多份
- (void)testPrintMultipleCopiesAndCount {
    // 最后一张&&最后一份打印完成
//    if ((self.pageIndex + 1 >= self.dataArray.count) && (self.printCount - 1 <= 0)) {
//        NSLog(@"打印完成....");
//        return;
//    }
//
//    // 先检查是否有下一张需要打印
//    self.pageIndex ++;
//
//    // 打印下一张
//    if (self.pageIndex < self.dataArray.count) {
//        [self printImage];
//        return;
//    }
//
//    // 检查是否有下一份需要打印
//    self.printCount --;
//    if (self.printCount > 0) {
//        // 从第一张开始打印
//        self.pageIndex = 0;
//        [self printImage];
//    }
}


#pragma mark - Private
- (NSArray *)funcNames {
    return @[
        @"打印图片",
        @"打印图组",
        @"固件升级",
        @"空白走纸",
        @"设置关机时间",
        @"2寸",
        @"3寸",
        @"4寸",
        @"纸张信息",
        @"固件版本",
        @"断开连接",
        @"耗材信息"
    ];
}

- (void)showConnectedPrinterInfo {
    if (!self.printer) return;
    unsigned int count;
    objc_property_t *properties = class_copyPropertyList([YKPrinter class], &count);
    NSString *string = @"当前连接的打印机信息：\n";
    for (int i = 0; i < count; i ++) {
        objc_property_t pro = properties[i];
        NSString *name = [NSString stringWithUTF8String:property_getName(pro)];
        NSNumber *num = (NSNumber *)[self.printer valueForKey:name];
        string = [string stringByAppendingString:[NSString stringWithFormat:@"  %@ = %@",name,num]];
    }
    
    self.printerInfoLabel.text = string;
}

- (UIImage *)scaleImage:(UIImage *)image andWidth:(NSInteger)width {
    NSInteger targetHeight = width * image.size.height / image.size.width;
    CGSize size;
    size.width = width;
    size.height = targetHeight;
    UIGraphicsBeginImageContext(size);
    [image drawInRect:CGRectMake(0, 0, width, targetHeight)];
    UIImage *scaledImage = UIGraphicsGetImageFromCurrentImageContext();
    UIGraphicsEndImageContext();
    
    return scaledImage;
}


- (void)writeDataToFile:(NSData *)contentData fileName:(NSString *)name {
    NSString *content = [[NSString alloc] initWithData:contentData encoding:NSUTF8StringEncoding];
    NSString *documentsDir = [NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) lastObject];
    documentsDir = [documentsDir stringByAppendingPathComponent:name];
    
    NSFileManager *fileManager = [NSFileManager defaultManager];
    BOOL isExit = [fileManager fileExistsAtPath:documentsDir];
    
    if (!isExit) {
        [fileManager createFileAtPath:documentsDir contents:nil attributes:nil];
    }
    
    BOOL res = [content writeToFile:documentsDir atomically:YES encoding:NSUTF8StringEncoding error:nil];
    if (res) {
        NSLog(@"INFO写入成功");
    }else {
        NSLog(@"INFO写入失败");
    }
}

@end
