//
//  YKImageProc.h
//  YKImageProc
//
//  Created by Sanyuer on 2019/7/29.
//  Copyright © 2019 Zhuhai YiKe Intelligent Technology Co., Ltd. All rights reserved.

#ifdef __APPLE__

#import <Foundation/Foundation.h>
#import <UIKit/UIKit.h>

//! Project version number for YKImageProcKit.
FOUNDATION_EXPORT double YKImageProcKitVersionNumber;

//! Project version string for YKImageProcKit.
FOUNDATION_EXPORT const unsigned char YKImageProcKitVersionString[];


// In this header, you should import all the public headers of your framework using statements like #import <YKImageProcKit/PublicHeader.h>

NS_ASSUME_NONNULL_BEGIN

typedef NS_ENUM(NSInteger, YKTextProc) {
    YKTextProc_Normal           = 0,    //正常处理
    YKTextProc_RemoveRed        = 1,    //去红
    YKTextProc_RemoveBlue       = 2,    //去蓝
    YKTextProc_RemoveRedBlue    = 3,    //去红蓝
    YKTextProc_RemoveColor      = 4,    //去彩
};

typedef NS_ENUM(NSInteger, YKImageEdge) {
    YKImageEdge_Left     = 0x01 << 0,
    YKImageEdge_Right    = 0x01 << 1,
    YKImageEdge_Top      = 0x01 << 2,
    YKImageEdge_Bottom   = 0x01 << 3,
};

typedef NS_ENUM(NSInteger, YKImageType) {
    YKImageTypeDOC     = 0, /// 文本模式
    YKImageTypeIMG    = 1, /// 图文模式
};


@interface YKImageProcKit : NSObject

+ (YKImageProcKit *)sharedInstance;

#pragma mark - process for printer
/// 图片打印 (灰度误差扩散)
/// @param srcImage 打印目标
- (UIImage *)printImageProc:(UIImage *)srcImage;

/// 图文打印 (灰度误差扩散)
/// @param srcImage 打印目标
- (UIImage *)printTextImageProc:(UIImage *)srcImage;

/// 文本打印 (普通全局二值化)
/// @param srcImage 打印目标
/// @param ithresh 阈值 [0-255]，范围外使用默认值127。
- (UIImage *)printTextProc:(UIImage *)srcImage threshold:(int)ithresh;

/// 文本打印 (niblack 二值化)
/// （字号过大会空心）
/// @param srcImage 打印目标
/// @param value [1,2, 3], 范围外使用1.
- (UIImage *)printTextProc:(UIImage *)srcImage boldValue:(int)value;

/// 文本打印 处理(openCV自适应局部阈值二值化)
/// （字号过大会空心）
/// @param srcImage 打印目标
/// @param opt 操作项
/// @param c 清晰度调节常量[0 - 50]，范围外使用默认值12。
/// @param value [1,2, 3], 范围外使用1.
- (UIImage *)printTextProc:(UIImage *)srcImage option:(YKTextProc)opt constant:(float)c boldValue:(int)value;

#pragma mark - process for preview

/// 去红去蓝去色（原图）
/// @param srcImage 预览目标
/// @param opt 操作项
- (UIImage *)previewImageProc:(UIImage *)srcImage option:(YKTextProc)opt NS_DEPRECATED_IOS(1_0_0, 2_5_4);

/// 长图分割
/// @param srcImage 分割目标
/// @param eheight 高度预设值
/// @return 分割点（高度）数组
- (nullable NSArray *)imageSegmentation:(UIImage *)srcImage heightEstimatedValue:(int)eheight;

/// 查找图片内容高度
/// @param srcImage 查找目标
/// @param spcHeight 留白高度
- (CGFloat)imageContentHeight:(UIImage *)srcImage space:(CGFloat)spcHeight;

/// 获取图片通道
- (NSInteger)imageChannel:(UIImage *)srcImage;

/// 图片灰度化
/// @param srcImage 目标图片
- (UIImage *)imageGrayscale:(UIImage *)srcImage;

/// 图片裁剪
/// @param srcImage 裁剪目标
- (UIImage *)imageCrop:(UIImage *)srcImage inRect:(CGRect)rect;

/// 图片压缩到指定尺寸
/// @param srcImage 压缩目标
/// @param size 目标尺寸
- (UIImage *)imageResize:(UIImage *)srcImage size:(CGSize)size;

/// 图片压缩到指定比例
/// @param srcImage 压缩目标
/// @param scale 压缩比例
- (UIImage *)imageResize:(UIImage *)srcImage scale:(CGFloat)scale;
/// 裁剪指定边
/// @param srcImage 裁剪目标
/// @param iedge 指定边
/// @param imargin 预留边距
- (UIImage *)imageCropEdge:(UIImage *)srcImage edge:(YKImageEdge)iedge margin:(uint32_t)imargin;

/// 裁剪四周白边
/// @param srcImage 裁剪目标
/// @param imargin 预留边距
- (UIImage *)imageCropAround:(UIImage *)srcImage margin:(uint32_t)imargin;

/// 图片拼接
/// @param images 图片组
/// @param spacing 间距
/// @param isHori 拼接方向
- (UIImage *)imageConcat:(NSArray<UIImage *> *)images spacing:(uint32_t)spacing isHorizontal:(BOOL)isHori;

#pragma mark - filter (leeyh)
/// 文字增强
/// @param erodeValue 腐蚀程度 demo值（1-5）
/// @param srcImage 目标图片
- (UIImage *)filterAdaptiveThresholdErode:(int)erodeValue image:(UIImage *)srcImage;

/// 文字增强（类似素描算法去背景）
/// @param srcImage 目标图片
/// @param isBlur 是否模糊，模糊后细节减少
/// @param erodeValue 腐蚀程度 demo值（1-5）
- (UIImage *)filterSketchReduceBackground:(UIImage *)srcImage blur:(BOOL)isBlur erode:(int)erodeValue;

/// 素描算法，保留线条
/// @param srcImage 目标图片
/// @param erodeValue 腐蚀程度 demo值（1-5）
- (UIImage *)filterSketchStreak:(UIImage *)srcImage erode:(int)erodeValue;

/// 直方图均衡化全通道
/// @param srcImage 目标图片
- (UIImage *)filterEqualizeHist:(UIImage *)srcImage;

/// 直方图均衡化单通道
/// @param srcImage 目标图片
- (UIImage *)filterEqualizeHistSingleChannel:(UIImage *)srcImage;

/// 对比度受限直方图均衡化
/// （通常用来增加许多图像的局部对比度，这种方法对于背景和前景都太亮或者太暗的图像非常有用）
/// @param srcImage 目标图片
/// @param value demo值 2 - 22
- (UIImage *)filterEqualizeHistClahe:(UIImage *)srcImage value:(int)value;

/// 拉普拉斯
/// （可以增强局部的图像对比度）
/// @param srcImage 目标图片
/// @param value demo值 5 - 9
- (UIImage *)filterEnhanceLaplace:(UIImage *)srcImage value:(int)value;

/// 对数变换
/// （整体对比度偏低并且灰度值偏低的图像增强效果较好）
/// @param srcImage 目标图片
/// @param value demo值 7 - 13
- (UIImage *)filterEnhanceLog:(UIImage *)srcImage value:(int)value;

/// 伽玛校正
/// （图像对比度偏低，并且整体亮度值偏高（对于于相机过曝）情况下的图像增强效果明显）
/// @param srcImage 目标图片
/// @param value demo值 0.001 - 7
- (UIImage *)filterEnhanceGamma:(UIImage *)srcImage value:(int)value;

#pragma mark - version 2.5.5后增加的
/**
 *  默认值情况下去红去蓝
 *  @param originImg 目标图片
 * @param option 图片类型
 */
- (UIImage *)removeColorWithImage:(UIImage *)originImg imageType:(YKImageType)option textProc:(YKTextProc)textProc NS_AVAILABLE_IOS(2_5_5);

/**
 *  图片去红
 * @param originImg 目标图片
 * @param option 图片类型
 * @param value 去红参数。参数越大，去除点数越少。范围(0,1] 默认值0.3
 */
- (UIImage *)removeRedWithImage:(UIImage *)originImg imageType:(YKImageType)option density:(float)value NS_AVAILABLE_IOS(2_5_5);

/**
 *  图片去蓝
 * @param originImg 目标图片
 * @param option 图片类型
 * @param value 去红参数。参数越大，去除点数越少。范围(0,1] 默认值0.3
 */
- (UIImage *)removeBlueWithImage:(UIImage *)originImg imageType:(YKImageType)option density:(float)value NS_AVAILABLE_IOS(2_5_5);


#pragma mark - test
- (UIImage *)testImage:(UIImage *)srcImage bsize:(int)bsize value:(float)value;

- (UIImage *)testImage2:(UIImage *)srcImage bsize:(int)bsize value:(float)value;

- (UIImage *)testImage3:(UIImage *)srcImage bsize:(int)bsize value:(float)value;

- (UIImage *)testImage4:(UIImage *)srcImage bsize:(int)bsize value:(float)value;

- (UIImage *)testImage5:(UIImage *)srcImage bsize:(int)bsize value:(float)value;

- (UIImage *)testImage6:(UIImage *)srcImage bsize:(int)bsize value:(float)value;

- (UIImage *)grayscaleTest:(UIImage *)srcImg bsize:(int)bsize value:(float)value;

- (UIImage *)binarizationTest:(UIImage *)srcImg bsize:(int)bsize value:(float)value;

- (UIImage *)errorDiffusionTest:(UIImage *)srcImg bsize:(int)bsize value:(float)value;

@end

NS_ASSUME_NONNULL_END

#endif //__APPLE__
