//
//  YKPrinter.h
//  YKPrinterKit
//
//  Created by YK on 2022/5/10.
//

#import <Foundation/Foundation.h>
#import <UIKit/UIKit.h>
#import <YKPrinterKit/YKPrinterConst.h>

NS_ASSUME_NONNULL_BEGIN

@interface YKPrinter : NSObject

#pragma mark ---------------------- Property ----------------------------
/** 蓝牙名字 */
@property (nonatomic, strong) NSString *name;

/** 蓝牙MAC地址 */
@property (nonatomic, strong) NSString *mac;

/** 信号强度 */
@property (nonatomic, strong) NSString *rssi;

/** UUID的description */
@property (nonatomic, strong) NSString *identifier;

/** 打印机类型 */
@property (nonatomic, assign) YKPrinterType printerType;

/** 打印机DPI */
@property (nonatomic, assign) YKPrinterWidthDot widthDot;

/** 打印机连接状态 */
@property (nonatomic, assign) YKPrinterConnectionState connectState;

/** 是否正在打印 */
@property (nonatomic, assign) BOOL isPrinting;

/** 是否正在升级 */
@property (nonatomic, assign) BOOL isUpgrading;

/** 是否正在充电 */
@property (nonatomic, assign) BOOL isCharging;

/** 是否开盖 */
@property (nonatomic, assign) BOOL isCoverOpend;

/** 是否电量过低 */
@property (nonatomic, assign) BOOL isLowBattery;

/** 是否过热 */
@property (nonatomic, assign) BOOL isOverHeat;

/** 是否缺纸 */
@property (nonatomic, assign) BOOL isNoPaper;

/** 电量百分比 */
@property (nonatomic, assign) NSInteger batteryPercentage;

/** 自动关机时间 单位min */
@property (nonatomic, assign) NSInteger offtime;

/** 打印速度  */
@property (nonatomic, assign) NSInteger speed;

/** 打印浓度 */
@property (nonatomic, assign) NSInteger density;

/** 固件版本 */
@property (nonatomic, strong) NSString *firVersion;

/** lp2-s 纸张有效性，无效不允许打印 */
@property (assign, nonatomic) BOOL isValidPaper;

/** 是否存在错误状态 例如：缺纸、开盖等 */
@property (assign, nonatomic, readonly, getter=isErrorState) BOOL errorState;


/** ---------------固件升级专用，外部不要使用---------------- */
/** updataStatus对应4个值，分别0、1、2、255
 * 0：设备收到了固件升级预备指令
 * 1：设备已经开始收固件数据了
 * 2：固件升级成功
 * 255：固件升级失败
 */
@property (nonatomic, assign) NSInteger updataStatus;
@property (nonatomic, assign) NSInteger updatePkgCount;

/** ------------T202型号需要的 --------------------  */

// 它不是用蓝牙来传输数据，仅用来配网
@property(strong, nonatomic) NSUUID *uuid;

// 是否有切刀
@property (assign, nonatomic) BOOL isHasCutter;

// 打印机是否繁忙
@property (assign, nonatomic) BOOL isBusy;

// 切刀错误
@property (assign, nonatomic) BOOL isCutterError;

// 故障一处
@property (assign, nonatomic) BOOL isDeviceFault;

// 其他错误
@property (assign, nonatomic) BOOL isOtherError;

// 按键走纸
@property (assign, nonatomic) BOOL isPressPaper;

// 打印机状态 0-空闲 1-云端打印中 2-app端打印中
@property (assign, nonatomic) NSInteger deviceStatus;

// 设备id
@property (strong, nonatomic) NSString *device_id;
@end

NS_ASSUME_NONNULL_END
